from __future__ import absolute_import
from __future__ import print_function

import sys
import os
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..', '..', '..')))

from pyros.rosinterface.mock import MockInterface
from pyros.rosinterface.mock.mockservice import statusecho_service, MockService

import nose
from nose.tools import timed, assert_true, assert_false, assert_equal, assert_raises


# @nose.SkipTest  # to help debugging ( FIXME : how to programmatically start only one test - maybe in fixture - ? )
@timed(5)  # this doesnt break if  hanging forever. need to replace with a breaking version
def test_mockinterface_update_services_c1():
    svc_names = ['/awesome_service', '/awesome_service_bis']
    mockif = MockInterface()

    for s in svc_names:
        assert_true(mockif.services.get(s) is None)  # services not exposed yet
    diffupdate = mockif.update_services(add_names=svc_names, remove_names=[])
    assert_false(diffupdate.added)  # service not detected cannot be added
    assert_false(diffupdate.removed)

    diffupdate = mockif.update_services(add_names=[], remove_names=svc_names)
    assert_false(diffupdate.added)
    assert_false(diffupdate.removed)  # service not added cannot be removed

    # simulating services appearing
    with mockif.mock_service(svc_names[0], statusecho_service), mockif.mock_service(svc_names[1], statusecho_service):
        mockif.services_args.update(svc_names)  # adding it to regex list to allow it to be exposed

        diffupdate = mockif.update_services(add_names=svc_names, remove_names=[])
        assert_false(diffupdate.removed)
        assert_true(diffupdate.added)  # service exposed can be added

        for s in svc_names:
            svc = mockif.services.get(s)
            assert_true(svc is not None)  # service exposed now
            assert_true(isinstance(svc, MockService))  # service type is MockService

        diffupdate = mockif.update_services(add_names=[], remove_names=svc_names)
        assert_false(diffupdate.added)
        assert_true(diffupdate.removed)  # service exposed can be deleted

        for s in svc_names:
            assert_true(mockif.services.get(s) is None)  # service not exposed anymore


# @nose.SkipTest  # to help debugging ( FIXME : how to programmatically start only one test - maybe in fixture - ? )
@timed(5)
def test_mockinterface_update_services_c2():
    svc_names = ['/awesome_service', '/awesome_service_bis']
    mockif = MockInterface()

    for s in svc_names:
        assert_true(mockif.services.get(s) is None)  # service not exposed yet
    diffupdate = mockif.update_services(add_names=svc_names, remove_names=[])
    assert_false(diffupdate.added)  # service not detected cannot be added
    assert_false(diffupdate.removed)

    diffupdate = mockif.update_services(add_names=[], remove_names=svc_names)
    assert_false(diffupdate.added)
    assert_false(diffupdate.removed)  # service not added cannot be removed

    # simulating service appearing
    with mockif.mock_service(svc_names[0], statusecho_service), mockif.mock_service(svc_names[1], statusecho_service):
        mockif.services_args.update(svc_names)  # adding it to regex list to allow it to be exposed

        diffupdate = mockif.update_services(add_names=svc_names, remove_names=[])
        assert_false(diffupdate.removed)
        assert_true(diffupdate.added)  # service exposed can be added

        for s in svc_names:
            svc = mockif.services.get(s)
            assert_true(svc is not None)  # service exposed now
            assert_true(isinstance(svc, MockService))  # service type is MockService

    for s in svc_names:
        svc = mockif.services.get(s)
        assert_true(svc is not None)  # service is still exposed even though it s gone from the system we interface to
        # WARNING : Using the service in this state will trigger errors.
        # These should be handled by the service class.
        # TODO : assert this

    diffupdate = mockif.update_services(add_names=[], remove_names=svc_names)
    assert_false(diffupdate.added)
    assert_true(diffupdate.removed)  # service non available (but added) can be deleted

    for s in svc_names:
        assert_true(mockif.services.get(s) is None)  # service not exposed


# @nose.SkipTest  # to help debugging ( FIXME : how to programmatically start only one test - maybe in fixture - ? )
@timed(5)
def test_mockinterface_expose_update_services_fullname():
    svc_names = ['/awesome_service', '/awesome_service_bis']
    mockif = MockInterface()

    for s in svc_names:
        assert_true(mockif.services.get(s) is None)  # service not exposed yet
    diffupdate = mockif.expose_services(svc_names)
    assert_false(diffupdate.added)  # service not detected cannot be added
    assert_false(diffupdate.removed)

    # simulating service appearing
    with mockif.mock_service(svc_names[0], statusecho_service), mockif.mock_service(svc_names[1], statusecho_service):

        diffupdate = mockif.expose_services(svc_names)
        assert_false(diffupdate.removed)
        assert_true(diffupdate.added)  # service available can be detected and be added

        for s in svc_names:
            svc = mockif.services.get(s)
            assert_true(svc is not None)  # service exposed now
            assert_true(isinstance(svc, MockService))  # service type is MockService

    for s in svc_names:
        svc = mockif.services.get(s)
        assert_true(svc is not None)  # service exposed now
        assert_true(isinstance(svc, MockService))  # service type is MockService

    # WARNING : Using the service in this state will trigger errors.
    # These should be handled by the service class.
    # TODO : assert this

    diffupdate = mockif.services_change_detect()
    assert_false(diffupdate.added)
    assert_true(diffupdate.removed)  # service lost can be detected and be removed

    for s in svc_names:
        assert_true(mockif.services.get(s) is None)  # service not exposed anymore

    diffupdate = mockif.expose_services(svc_names)
    assert_false(diffupdate.removed)
    assert_false(diffupdate.added)  # new expose call doesn't change anything

    for s in svc_names:
        assert_true(mockif.services.get(s) is None)  # service not exposed anymore


# @nose.SkipTest  # to help debugging ( FIXME : how to programmatically start only one test - maybe in fixture - ? )
@timed(5)
def test_mockinterface_update_expose_services_fullname():
    svc_names = ['/awesome_service', '/awesome_service_bis']
    mockif = MockInterface()

    diffupdate = mockif.services_change_detect()
    assert_false(diffupdate.added)  # service not available is not detected and not added
    assert_false(diffupdate.removed)  # service not added previously is not removed

    with mockif.mock_service(svc_names[0], statusecho_service), mockif.mock_service(svc_names[1], statusecho_service):

        diffupdate = mockif.services_change_detect()
        assert_false(diffupdate.removed)
        assert_false(diffupdate.added)  # service available is not detected and added without previous expose call

        for s in svc_names:
            svc = mockif.services.get(s)
            assert_true(svc is None)  # service not exposed now

        diffupdate = mockif.expose_services(svc_names)
        assert_false(diffupdate.removed)
        assert_true(diffupdate.added)  # new expose call add the service because it is already available

        for s in svc_names:
            svc = mockif.services.get(s)
            assert_true(svc is not None)  # service exposed now
            assert_true(isinstance(svc, MockService))  # service type is MockService

        diffupdate = mockif.services_change_detect()
        assert_false(diffupdate.removed)
        assert_false(diffupdate.added)  # new detection call doesnt change anything

        for s in svc_names:
            svc = mockif.services.get(s)
            assert_true(svc is not None)  # service still exposed
            assert_true(isinstance(svc, MockService))  # service type is still MockService

        diffupdate = mockif.expose_services([])
        assert_false(diffupdate.added)
        assert_true(diffupdate.removed)  # new expose call can remove the service even if it is still available

        for s in svc_names:
            assert_true(mockif.services.get(s) is None)  # service not exposed anymore

        diffupdate = mockif.expose_services(svc_names)
        assert_false(diffupdate.removed)
        assert_true(diffupdate.added)  # new expose call can readd if it is still available

    for s in svc_names:
        svc = mockif.services.get(s)
        assert_true(svc is not None)  # service exposed now
        assert_true(isinstance(svc, MockService))  # service type is MockService

    # WARNING : Using the service in this state will trigger errors.
    # These should be handled by the service class.
    # TODO : assert this

    diffupdate = mockif.expose_services([])
    assert_false(diffupdate.added)
    assert_true(diffupdate.removed)  # new expose call can remove the service even if it is not available

    for s in svc_names:
        assert_true(mockif.services.get(s) is None)  # service not exposed anymore

    diffupdate = mockif.services_change_detect()
    assert_false(diffupdate.added)  # no appeared service : nothing is added
    assert_false(diffupdate.removed)  # disappeared service was already removed

    for s in svc_names:
        assert_true(mockif.services.get(s) is None)  # service not exposed anymore


# @nose.SkipTest  # to help debugging ( FIXME : how to programmatically start only one test - maybe in fixture - ? )
@timed(5)
def test_mockinterface_expose_services_regex():
    svc_names = ['/awesome_service', '/awesome_service_bis']
    svc_regex = '/awesome.*'
    mockif = MockInterface()

    for s in svc_names:
        assert_true(mockif.services.get(s) is None)  # service not exposed yet
    diffupdate = mockif.expose_services([svc_regex])
    assert_false(diffupdate.added)  # service not detected cannot be added
    assert_false(diffupdate.removed)

    with mockif.mock_service(svc_names[0], statusecho_service), mockif.mock_service(svc_names[1], statusecho_service):

        diffupdate = mockif.services_change_detect()
        assert_false(diffupdate.removed)
        assert_true(diffupdate.added)  # new detection call finds the service and adds it

        for s in svc_names:
            svc = mockif.services.get(s)
            assert_true(svc is not None)  # service exposed
            assert_true(isinstance(svc, MockService))  # service type is MockService

    diffupdate = mockif.services_change_detect()
    assert_false(diffupdate.added)
    assert_true(diffupdate.removed)  # new detection call finds the service and removes it

    for s in svc_names:
        assert_true(mockif.services.get(s) is None)  # service not exposed anymore


# @nose.SkipTest  # to help debugging ( FIXME : how to programmatically start only one test - maybe in fixture - ? )
@timed(5)
def test_mockinterface_update_expose_services_fullname_diff():
    svc_names = ['/awesome_service', '/awesome_service_bis']
    mockif = MockInterface()

    diffupdate = mockif.services_change_diff([], [])
    assert_false(diffupdate.added)  # service not passed in diff is not detected and not added
    assert_false(diffupdate.removed)  # service not added previously is not removed

    with mockif.mock_service(svc_names[0], statusecho_service), mockif.mock_service(svc_names[1], statusecho_service):

        diffupdate = mockif.services_change_diff(svc_names, [])
        assert_false(diffupdate.removed)
        assert_false(diffupdate.added)  # service available is not detected and added without previous expose call

        for s in svc_names:
            svc = mockif.services.get(s)
            assert_true(svc is None)  # service not exposed now

        diffupdate = mockif.expose_services(svc_names)
        assert_false(diffupdate.removed)
        assert_true(diffupdate.added)  # new expose call add the service because it is already available

        for s in svc_names:
            svc = mockif.services.get(s)
            assert_true(svc is not None)  # service exposed now
            assert_true(isinstance(svc, MockService))  # service type is MockService

        diffupdate = mockif.services_change_diff([], [])
        assert_false(diffupdate.removed)
        assert_false(diffupdate.added)  # empty diff call doesnt change anything

        for s in svc_names:
            svc = mockif.services.get(s)
            assert_true(svc is not None)  # service still exposed
            assert_true(isinstance(svc, MockService))  # service type is still MockService

        diffupdate = mockif.expose_services([])
        assert_false(diffupdate.added)
        assert_true(diffupdate.removed)  # new expose call can remove the service even if it is still available

        for s in svc_names:
            assert_true(mockif.services.get(s) is None)  # service not exposed anymore

        diffupdate = mockif.expose_services(svc_names)
        assert_false(diffupdate.removed)
        assert_true(diffupdate.added)  # new expose call can readd if it is still available

    for s in svc_names:
        svc = mockif.services.get(s)
        assert_true(svc is not None)  # service exposed now
        assert_true(isinstance(svc, MockService))  # service type is MockService

    # WARNING : Using the service in this state will trigger errors.
    # These should be handled by the service class.
    # TODO : assert this

    diffupdate = mockif.expose_services([])
    assert_false(diffupdate.added)
    assert_true(diffupdate.removed)  # new expose call can remove the service even if it is not available

    for s in svc_names:
        assert_true(mockif.services.get(s) is None)  # service not exposed anymore

    diffupdate = mockif.services_change_diff([], [])
    assert_false(diffupdate.added)  # no service passed in diff : nothing is added
    assert_false(diffupdate.removed)  # disappeared service was already removed

    for s in svc_names:
        assert_true(mockif.services.get(s) is None)  # service not exposed anymore


# @nose.SkipTest  # to help debugging ( FIXME : how to programmatically start only one test - maybe in fixture - ? )
@timed(5)
def test_mockinterface_expose_services_regex_diff():
    svc_names = ['/awesome_service', '/awesome_service_bis']
    svc_regex = '/awesome.*'
    mockif = MockInterface()

    for s in svc_names:
        assert_true(mockif.services.get(s) is None)  # service not exposed yet
    diffupdate = mockif.expose_services([svc_regex])
    assert_false(diffupdate.added)  # service not detected cannot be added
    assert_false(diffupdate.removed)

    with mockif.mock_service(svc_names[0], statusecho_service), mockif.mock_service(svc_names[1], statusecho_service):

        diffupdate = mockif.services_change_diff(svc_names, [])
        assert_false(diffupdate.removed)
        assert_true(diffupdate.added)  # new diff call finds the service and adds it

        for s in svc_names:
            svc = mockif.services.get(s)
            assert_true(svc is not None)  # service exposed
            assert_true(isinstance(svc, MockService))  # service type is MockService

    diffupdate = mockif.services_change_diff([], svc_names)
    assert_false(diffupdate.added)
    assert_true(diffupdate.removed)  # new diff call finds the service and removes it

    for s in svc_names:
        assert_true(mockif.services.get(s) is None)  # service not exposed anymore

#TODO : test exception raised properly when update transient cannot happen


# TODO : Same for topics and params

if __name__ == '__main__':

    import nose
    nose.runmodule()







from __future__ import absolute_import

import sys
import os

# This is needed if running this test directly (without using nose loader)
# prepending because ROS relies on package dirs list in PYTHONPATH and not isolated virtualenvs
# And we need our current module to be found first.
current_path = os.path.abspath(os.path.join(os.path.dirname(__file__), '..', '..'))
# if not current_path in sys.path:
sys.path.insert(1, current_path)  # sys.path[0] is always current path as per python spec

from pyros.pyros_mock import PyrosMock
from pyros.pyros_client import PyrosClient

import nose
from nose.tools import assert_equal, assert_raises


class TestPyrosClientOnMock(object):
    def setUp(self):
        self.mockInstance = PyrosMock()
        # setting up mock instance
        cmd_conn = self.mockInstance.start()
        self.client = PyrosClient(cmd_conn)

    def tearDown(self):
        self.mockInstance.shutdown()

    ### TOPICS ###

    # TODO : test list features more !
    def test_list_all(self):
        t = self.client.topics()
        # Make sure we get all mock topics
        assert t is not None


    def test_inject_None(self):  # injecting None is meaningless and should return false
        assert self.client.topic_inject('random_topic', None)  # simply check that it injected (default Empty since we support kwargs)

    def test_inject_Empty(self):
        assert self.client.topic_inject('random_topic')  # simply check if injected

    #TODO : how to test strict backend with Mock ?
    #def test_inject_Wrong(self):
    #    with assert_raises(Exception) as expt:  # TODO : be more specific
    #        data = 42
    #        self.client.topic_inject('random_topic', data)  # simply check exception raised
    #    # assert_equal(expt, smthg...)

    def test_extract_None(self):
        assert self.client.topic_extract('random_topic') is None  # simply check if nothing extracted

    def test_inject_extract_echo_Empty(self):
        assert self.client.topic_inject('random_topic')  # default should be {}
        print "injected message content {0}".format({})
        recv = self.client.topic_extract('random_topic')
        print "extracted message {0}".format(recv)
        assert recv == {}

    def test_inject_extract_echo_Simple_Arg(self):
        data = 'data_string'
        assert self.client.topic_inject('random_topic', data)
        print "injected message content {0}".format(data)
        recv = self.client.topic_extract('random_topic')
        print "extracted message content {0}".format(recv)
        assert recv == data

    def test_inject_extract_echo_Complex_Arg(self):
        data = {'first': 'first_string', 'second': 'second_string'}
        assert self.client.topic_inject('random_topic', data)
        print "injected message content {0}".format(data)
        recv = self.client.topic_extract('random_topic')
        print "extracted message content {0}".format(recv)
        assert recv == data

    def test_inject_extract_echo_Simple_KWArgs(self):
        assert self.client.topic_inject('random_topic', data='data_string')
        print "injected message content {0}".format("data='data_string'")
        recv = self.client.topic_extract('random_topic')
        print "extracted message content {0}".format(recv)
        assert recv == {'data':'data_string'}

    def test_inject_extract_echo_Complex_KWArgs(self):
        assert self.client.topic_inject('random_topic', first='first_string', second='second_string')
        print "injected message content {0}".format("first='first_string', second='second_string'")
        recv = self.client.topic_extract('random_topic')
        print "extracted message content {0}".format(recv)
        assert recv == {'first': 'first_string', 'second': 'second_string'}

    ### SERVICES ###
    # TODO : think how to test strict backend with Mock ?
    #def test_call_Wrong(self):
    #    with assert_raises(Exception) as expt:  # TODO : be more specific
    #        data = 42
    #        print "request content {0}".format(data)
    #        resp = self.client.service_call('random_service', data)  # simply check exception raised
    #    # assert_equal(expt, smthg...)

    def test_call_echo_Simple_Arg(self):
        data = 'data_string'
        print "request content {0}".format(data)
        resp = self.client.service_call('random_service', data)
        print "response content {0}".format(resp)
        assert resp == data

    def test_call_echo_None(self):
        print "request content {0}".format({})
        resp = self.client.service_call('random_service', None)  # calling with None is interpreted as default (Empty) call (since we support kwargs)
        print "response content {0}".format(resp)
        assert resp == {}

    def test_call_echo_Empty(self):
        print "request content {0}".format({})
        resp = self.client.service_call('random_service')
        print "response content {0}".format(resp)
        assert resp == {}

    def test_call_echo_Complex_Arg(self):
        data = {'first': 'first_string', 'second': 'second_string'}
        print "request content {0}".format(data)
        resp = self.client.service_call('random_service', data)
        print "response content {0}".format(resp)
        assert resp == data

    def test_call_echo_Simple_KWArgs(self):
        print "request content {0}".format("data='data_string'")
        resp = self.client.service_call('random_service', data='data_string')
        print "response content {0}".format(resp)
        assert resp == {'data': 'data_string'}

    def test_call_echo_Complex_KWArgs(self):
        print "injected message content {0}".format("first='first_string', second='second_string'")
        resp = self.client.service_call('random_service', first='first_string', second='second_string')
        print "extracted message content {0}".format(resp)
        assert resp == {'first': 'first_string', 'second': 'second_string'}

    ### PARAMS ###
    #def test_set_None(self):  # injecting None is meaningless and should return false
    #    assert not self.client.param_set('random_param', None)  # simply check that it didnt set

    def test_set_Empty(self):
        assert self.client.param_set('random_param')  # simply check if set

    def test_get_None(self):
        assert self.client.param_get('random_param') is None  # simply check if nothing got

    def test_set_get_echo_Empty(self):
        assert self.client.param_set('random_param')  # default should be {}
        print "set value content {0}".format({})
        recv = self.client.param_get('random_param')
        print "got value content {0}".format(recv)
        assert recv == {}

    def test_set_get_echo_Simple_Arg(self):
        data = 'data_string'
        assert self.client.param_set('random_param', data)
        print "set value content {0}".format(data)
        recv = self.client.param_get('random_param')
        print "got value content {0}".format(recv)
        assert recv == data

    def test_set_get_echo_Complex_Arg(self):
        data = {'first': 'first_string', 'second': 'second_string'}
        assert self.client.param_set('random_param', data)
        print "set value content {0}".format(data)
        recv = self.client.param_get('random_param')
        print "got value content {0}".format(recv)
        assert recv == data

    def test_set_get_echo_Simple_KWArgs(self):
        assert self.client.param_set('random_param', data='data_string')
        print "set value content {0}".format("data='data_string'")
        recv = self.client.param_get('random_param')
        print "got value content {0}".format(recv)
        assert recv == {'data':'data_string'}

    def test_set_get_echo_Complex_KWArgs(self):
        assert self.client.param_set('random_param', first='first_string', second='second_string')
        print "set value content {0}".format("first='first_string', second='second_string'")
        recv = self.client.param_get('random_param')
        print "got value content {0}".format(recv)
        assert recv == {'first': 'first_string', 'second': 'second_string'}

# TODO test service that throw exception


from __future__ import absolute_import, print_function

import sys
import os
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..', '..', '..')))

from pyros.rosinterface.mock import extract_values, populate_instance, FieldTypeMismatchException, NonexistentFieldException, StatusMsg
from nose.tools import assert_true, assert_false, assert_raises


def test_populate_msg_bool_true():
    msg = populate_instance(True, bool())
    print("msg is of type {0}".format(type(msg)))
    assert_true(isinstance(msg, bool))
    assert_true(msg)


def test_populate_msg_bool_false():
    msg = populate_instance(False, bool())
    print("msg is of type {0}".format(type(msg)))
    assert_true(isinstance(msg, bool))
    assert_false(msg)


def test_populate_msg_int_to_int():
    msg = populate_instance(42, int())
    print("msg is of type {0}".format(type(msg)))
    assert_true(isinstance(msg, int))
    assert_true(msg == 42)


def test_populate_msg_int_to_float():
    msg = populate_instance(42, float())
    print("msg is of type {0}".format(type(msg)))
    assert_true(isinstance(msg, float))
    assert_true(msg == 42.0)


def test_populate_msg_float():
    msg = populate_instance(3.1415, float())
    print("msg is of type {0}".format(type(msg)))
    assert_true(isinstance(msg, float))
    assert_true(msg == 3.1415)


def test_populate_msg_float_to_int_error():
    with assert_raises(FieldTypeMismatchException):
        populate_instance(3.1415, int())


def test_populate_msg_str_to_str():
    msg = populate_instance(r'forty two', str())
    print("msg is of type {0}".format(type(msg)))
    assert_true(isinstance(msg, str))
    assert_true(msg == r'forty two')


def test_populate_msg_str_to_unicode():
    msg = populate_instance(r'forty two', unicode())
    print("msg is of type {0}".format(type(msg)))
    assert_true(isinstance(msg, unicode))
    assert_true(msg == u'forty two')


def test_populate_msg_unicode_to_unicode():
    msg = populate_instance(u'forty two', unicode())
    print("msg is of type {0}".format(type(msg)))
    assert_true(isinstance(msg, unicode))
    assert_true(msg == u'forty two')


def test_populate_msg_unicode_to_str_error():
    with assert_raises(FieldTypeMismatchException):
        populate_instance(u'forty two', str())


def test_populate_msg_list():
    msg = populate_instance([False, 42, 3.1415, r'fortytwo', u'forty two'], list())
    print("msg is of type {0}".format(type(msg)))
    assert_true(isinstance(msg, list))
    assert_true(msg == [False, 42, 3.1415, r'fortytwo', u'forty two'])


def test_populate_msg_list_to_tuple_error():
    with assert_raises(FieldTypeMismatchException):
        populate_instance([False, 42, 3.1415, r'fortytwo', u'forty two'], tuple())


def test_populate_msg_tuple_to_tuple():
    msg = populate_instance((False, 42, 3.1415, r'fortytwo', u'forty two'), tuple())
    print("msg is of type {0}".format(type(msg)))
    assert_true(isinstance(msg, tuple))
    assert_true(msg == (False, 42, 3.1415, r'fortytwo', u'forty two'))


def test_populate_msg_tuple_to_list():
    msg = populate_instance((False, 42, 3.1415, r'fortytwo', u'forty two'), list())
    print("msg is of type {0}".format(type(msg)))
    assert_true(isinstance(msg, list))
    assert_true(msg == [False, 42, 3.1415, r'fortytwo', u'forty two'])


def test_populate_msg_dict_to_status():
    msg = populate_instance(
        {"error": True, "code": 7, "message": "Actual Error"},
        StatusMsg(error=False, code=42, message="Not an error")
    )
    print("msg is of type {0}".format(type(msg)))
    assert_true(isinstance(msg, StatusMsg))
    assert_true(msg.error)
    assert_true(msg.code == 7)
    assert_true(msg.message == "Actual Error")


def test_populate_msg_dict_to_status_error():

    with assert_raises(NonexistentFieldException):
        msg = populate_instance(
            {"error": True, "code": 7, "message": "Actual Error", "non-existent": "field"},
            StatusMsg(error=False, code=42, message="Not an error")
        )


    ###### EXTRACT #######

def test_extract_msg_bool_true():
    msg = populate_instance(True, bool())
    data = extract_values(msg)
    assert_true(isinstance(data, bool))
    assert_true(data)


def test_extract_msg_bool_false():
    msg = populate_instance(False, bool())
    data = extract_values(msg)
    assert_true(isinstance(data, bool))
    assert_false(data)


def test_extract_msg_int_to_int():
    msg = populate_instance(42, int())
    data = extract_values(msg)
    assert_true(isinstance(data, int))
    assert_true(data == 42)


def test_extract_msg_int_to_float():
    msg = populate_instance(42, float())
    data = extract_values(msg)
    assert_true(isinstance(data, float))
    assert_true(data == 42.0)


def test_extract_msg_float():
    msg = populate_instance(3.1415, float())
    data = extract_values(msg)
    assert_true(isinstance(data, float))
    assert_true(data == 3.1415)


def test_extract_msg_str_to_str():
    msg = populate_instance(r'forty two', str())
    data = extract_values(msg)
    assert_true(isinstance(data, str))
    assert_true(data == r'forty two')


def test_extract_msg_str_to_unicode():
    msg = populate_instance(r'forty two', unicode())
    data = extract_values(msg)
    assert_true(isinstance(data, unicode))
    assert_true(data == u'forty two')


def test_extract_msg_unicode_to_unicode():
    msg = populate_instance(u'forty two', unicode())
    data = extract_values(msg)
    assert_true(isinstance(data, unicode))
    assert_true(data == u'forty two')


def test_extract_msg_list():
    msg = populate_instance([False, 42, 3.1415, r'fortytwo', u'forty two'], list())
    data = extract_values(msg)
    assert_true(isinstance(data, list))
    assert_true(data == [False, 42, 3.1415, r'fortytwo', u'forty two'])


def test_extract_msg_tuple_to_tuple():
    msg = populate_instance((False, 42, 3.1415, r'fortytwo', u'forty two'), tuple())
    data = extract_values(msg)
    assert_true(isinstance(data, tuple))
    assert_true(data == (False, 42, 3.1415, r'fortytwo', u'forty two'))


def test_extract_msg_tuple_to_list():
    msg = populate_instance((False, 42, 3.1415, r'fortytwo', u'forty two'), list())
    data = extract_values(msg)
    assert_true(isinstance(msg, list))
    assert_true(msg == [False, 42, 3.1415, r'fortytwo', u'forty two'])


def test_extract_msg_dict_to_status():
    msg = populate_instance(
        {"error": True, "code": 7, "message": "Actual Error"},
        StatusMsg(error=False, code=42, message="Not an error")
    )
    data = extract_values(msg)
    assert_true(isinstance(data, StatusMsg))
    assert_true(data.error)
    assert_true(data.code == 7)
    assert_true(data.message == "Actual Error")

if __name__ == '__main__':

    import nose
    nose.runmodule()



from __future__ import absolute_import
from __future__ import print_function

import logging

from .api import rospy_safe as rospy

# create logger
_logger = logging.getLogger(__name__)
# and let it propagate to parent logger, or other handler
# the user of pyros should configure handlers

from .param_if import ParamBack, ParamTuple

from ..baseinterface import TransientIfPool, DiffTuple


class RosParamIfPool(TransientIfPool):

    """
    MockInterface.
    """
    def __init__(self, params=None):
        # This base constructor assumes the system to interface with is already available ( can do a get_svc_available() )
        super(RosParamIfPool, self).__init__(params, transients_desc="parameters")

    # mock functions that simulate/mock similar interface than what is found on multiprocess framework supported
    # We should try our best to go for the lowest common denominator here
    # PARAMS
    def get_transients_available(self):  # function returning all params available on the system
        return self.available

    def transient_type_resolver(self, param_name):  # function resolving the type of a param
        prm = self.available.get(param_name)
        if prm:
            if prm.type is None:  # if the type is unknown, lets discover it (since the param is supposed to exist)
                try:
                    prm.type = type(rospy.get_param(param_name))  # we use the detected python type here (since there is no rospy param type interface for this)
                except KeyError:  # exception can occur -> just reraise
                    raise
            return prm.type  # return the first we find. enough.
        else:
            rospy.logerr("ERROR while resolving {param_name}. Param not known as available. Ignoring".format(**locals()))
            return None

    def TransientMaker(self, param_name, param_type, *args, **kwargs):  # the param class implementation
        return ParamBack(param_name, param_type, *args, **kwargs)

    def TransientCleaner(self, param):  # the param class implementation
        return param.cleanup()

    ##bwcompat
    # REQUESTED
    @property
    def params_args(self):
        return self.transients_args

    # AVAILABLE
    @property
    def params_available(self):
        return self.available

    # INTERFACED
    @property
    def params(self):
        return self.transients

    # EXPOSE
    def expose_params(self, prm_regex):
        return self.expose_transients_regex(prm_regex)

    def get_param_available(self):  # function returning all params available on the system
        return self.get_transients_available()

    def param_type_resolver(self, param_name):  # function resolving the type of a param
        prm = self.transient_type_resolver(param_name)

    def ParamMaker(self, param_name, param_type):  # the param class implementation
        return self.TransientMaker(param_name, param_type)

    def ParamCleaner(self, param):  # the param class implementation
        return self.TransientCleaner(param)
    ##############3

    def retrieve_state(self):
        # TODO : maybe not here ? (to match services and topic behavior... and unify the request)
        """
        called to update params from rospy.
        CAREFUL : this can be called from another thread (subscriber callback)
        """
        params = rospy.get_param_names()
        self.reset_state(params)

    def reset_state(self, params):
        """
        called to update params from rospy.
        CAREFUL : this can be called from another thread (subscriber callback)
        """
        self.available = dict()
        for p in params:
            pt = []
            ptp = ParamTuple(name=p, type=pt[1] if len(pt) > 0 else None)
            self.available[ptp.name] = ptp

    def compute_state(self, params_dt):
        """
        called to update params from rospy.
        CAREFUL : this can be called from another thread (subscriber callback)
        """

        computed_params_dt = DiffTuple([], [])
        for p in params_dt.added:
            pt = ParamTuple(name=p, type=None)
            if pt.name in self.available:
                if self.available[pt.name].type is None or pt.type is not None:
                    self.available[pt.name].type = pt.type
            else:
                self.available[pt.name] = pt
                computed_params_dt.added.append(pt.name)

        for p in params_dt.removed:
            pt = ParamTuple(name=p, type=None)
            if pt.name in self.available:
                self.available.pop(pt.name, None)
                computed_params_dt.removed.append(pt.name)

        return computed_params_dt


    # for use with line_profiler or memory_profiler
    # Not working yet... need to solve multiprocess profiling issues...
    #@profile
    def update_delta(self, params_dt):

        # First we need to reflect the external system state in internal cache
        computed_params_dt = self.compute_state(params_dt)

        if computed_params_dt.added or computed_params_dt.removed:
            _logger.debug(
                rospy.get_name() + " Params Delta {params_dt}".format(**locals()))

        # _logger.debug("Params ADDED : {0}".format([p for p in available_dt.added]))
        # _logger.debug("Params GONE : {0}".format([p for p in available_dt.removed]))

        # Second we update our interfaces based on that system state difference
        dt = self.transient_change_diff(
            transient_appeared=computed_params_dt.added,
            transient_gone=computed_params_dt.removed
        )

        if dt.added or dt.removed:
            _logger.debug(rospy.get_name() + " Update Delta {dt}".format(**locals()))
        return dt

    # for use with line_profiler or memory_profiler
    # Not working yet... need to solve multiprocess profiling issues...
    #@profile
    def update(self, params):

        # First we need to reflect the external system state in internal cache
        self.reset_state(params)

        if params:
            _logger.debug(
                rospy.get_name() + " Params List {params}".format(**locals()))

        # Second we update our interfaces based on that new system state
        # TODO : pass full params state here to avoid having to retrieve indirectly
        dt = self.transient_change_detect()

        if dt.added or dt.removed:
            _logger.debug(rospy.get_name() + " Update Delta {dt}".format(**locals()))
        return dt


TransientIfPool.register(RosParamIfPool)



from __future__ import absolute_import
from __future__ import print_function

import os
import sys

# This is needed if running this test directly (without using nose loader)
# prepending because ROS relies on package dirs list in PYTHONPATH and not isolated virtualenvs
# And we need our current module to be found first.
current_path = os.path.abspath(os.path.join(os.path.dirname(__file__), '..', '..'))
# if not current_path in sys.path:
sys.path.insert(1, current_path)  # sys.path[0] is always current path as per python spec

import time

from pyros.rosinterface.mock.mocksystem import (
    mock_publisher_remote, mock_subscriber_remote, mock_service_remote, mock_param_remote,
    services_available_remote, services_available_type_remote,
    topics_available_remote, topics_available_type_remote,
    params_available_remote, params_available_type_remote,
)
from pyros import PyrosMock
from pyros.rosinterface.mock.mocksystem import statusecho_topic
import pyzmp
from nose.tools import timed, assert_true, assert_false, assert_equal, assert_raises


### TESTING NODE CREATION / TERMINATION ###
# @nose.SkipTest  # to help debugging ( FIXME : how to programmatically start only one test - maybe in fixture - ? )
@timed(5)
def test_mocknode_creation_termination():
    mockn = PyrosMock()
    assert_false(mockn.is_alive())
    mockn.start()
    try:
        assert_true(mockn.is_alive())
    finally:
        mockn.shutdown()
        assert_false(mockn.is_alive())


@timed(5)
def test_mocknode_provide_services():  # Here we check that this node actually provides all the services
    mockn = PyrosMock()
    assert_false(mockn.is_alive())

    assert_true(hasattr(mockn, 'msg_build'))
    assert_true(hasattr(mockn, 'topic'))
    assert_true(hasattr(mockn, 'topics'))
    assert_true(hasattr(mockn, 'service'))
    assert_true(hasattr(mockn, 'services'))
    assert_true(hasattr(mockn, 'param'))
    assert_true(hasattr(mockn, 'params'))
    assert_true(hasattr(mockn, 'setup'))

    mockn.start()
    try:
        assert_true(mockn.is_alive())

        print("Discovering msg_build Service...")
        msg_build = pyzmp.discover("msg_build", 5)  # we wait a bit to let it time to start
        assert_false(msg_build is None)
        assert_equal(len(msg_build.providers), 1)

        print("Discovering topic Service...")
        topic = pyzmp.discover("topic", 5)  # we wait a bit to let it time to start
        assert_false(topic is None)
        assert_equal(len(topic.providers), 1)

        print("Discovering topics Service...")
        topic_list = pyzmp.discover("topics", 5)  # we wait a bit to let it time to start
        assert_false(topic_list is None)
        assert_equal(len(topic_list.providers), 1)

        print("Discovering service Service...")
        service = pyzmp.discover("service", 5)  # we wait a bit to let it time to start
        assert_false(service is None)
        assert_equal(len(service.providers), 1)

        print("Discovering services Service...")
        service_list = pyzmp.discover("services", 5)  # we wait a bit to let it time to start
        assert_false(service_list is None)
        assert_equal(len(service_list.providers), 1)

        print("Discovering param Service...")
        param = pyzmp.discover("param", 5)  # we wait a bit to let it time to start
        assert_false(param is None)
        assert_equal(len(param.providers), 1)

        print("Discovering params Service...")
        param_list = pyzmp.discover("params", 5)  # we wait a bit to let it time to start
        assert_false(param_list is None)
        assert_equal(len(param_list.providers), 1)

        print("Discovering setup Service...")
        param_list = pyzmp.discover("setup", 5)  # we wait a bit to let it time to start
        assert_false(param_list is None)
        assert_equal(len(param_list.providers), 1)
    finally:
        mockn.shutdown()
        assert_false(mockn.is_alive())


def test_mocknode_publishers_detect():  # Here we check that this node actually detects a topic
    mockn = PyrosMock(kwargs={
        'services': [],
        'publishers': ['test_topic'],
        'subscribers': [],
        'params': []
    })
    assert_false(mockn.is_alive())

    assert_true(hasattr(mockn, 'topics'))

    # starting the node
    mockn.start()

    # checking interface is still None here ( instantiated in child only )
    assert_true(mockn.interface is None)

    # Services are initialized in run() method of pyzmp.Node, after interface has been initialized
    try:
        assert_true(mockn.is_alive())

        with mock_publisher_remote('test_topic', statusecho_topic):

            # asserting the mock system has done its job from our point of view at least
            assert_true('test_topic' in topics_available_remote)
            assert_equal(topics_available_type_remote['test_topic'], statusecho_topic)

            # Getting topics list from child process
            print("Discovering topics Service...")
            topics = pyzmp.discover("topics", 3)  # we wait a bit to let it time to start
            assert_false(topics is None)
            assert_equal(len(topics.providers), 1)

            time.sleep(mockn.update_interval + 1)  # make sure we let update time to kick in

            res = topics.call(recv_timeout=6000000)
            # the mock system should have done its job from the other process perspective too
            # via multiprocess manager list
            assert_true('test_topic' in res)  # topic detected since in list of exposed topics

    finally:
        mockn.shutdown()
        assert_false(mockn.is_alive())


def test_mocknode_publishers_detect_setup():  # Here we check that this node actually detects a topic upon setup
    mockn = PyrosMock()
    assert_false(mockn.is_alive())

    assert_true(hasattr(mockn, 'topics'))

    mockn.start()
    try:
        assert_true(mockn.is_alive())

        with mock_publisher_remote('test_topic', statusecho_topic):

            print("Discovering topics Service...")
            topics = pyzmp.discover("topics", 3)  # we wait a bit to let it time to start
            assert_false(topics is None)
            assert_equal(len(topics.providers), 1)

            res = topics.call()
            assert_true(not 'test_topic' in res)  # topic not detected since not in list of exposed topics

            print("Discovering setup Service...")
            setup = pyzmp.discover("setup", 3)  # we wait a bit to let it time to start
            assert_false(setup is None)
            assert_equal(len(setup.providers), 1)

            setup.call(kwargs={'services': [], 'topics': ['test_topic'], 'params': []})

            time.sleep(mockn.update_interval + 1)  # waiting for update to kick in

            res = topics.call()
            assert_true('test_topic' in res)
    finally:
        mockn.shutdown()
        assert_false(mockn.is_alive())


def test_mocknode_publishers_detect_throttled():
    """
    Testing that the mocknode detection of topics is throttled properly
    :return:
    """
    mockn = PyrosMock()
    assert_false(mockn.is_alive())

    assert_true(hasattr(mockn, 'topics'))

    mockn.update_interval = 5  # we wait 5 seconds between each update_throttled call
    mockn.start()  # one update will be triggered, and then nothing for the next 10 seconds
    try:
        assert_true(mockn.is_alive())

        print("Discovering setup Service...")
        setup = pyzmp.discover("setup", 3)  # we wait a bit to let it time to start
        assert_false(setup is None)
        assert_equal(len(setup.providers), 1)

        setup.call(kwargs={'services': [], 'topics': ['test_topic'], 'params': []})

        with mock_publisher_remote('test_topic', statusecho_topic):

            print("Discovering topics Service...")
            topics = pyzmp.discover("topics", 3)  # we wait a bit to let it time to start
            assert_false(topics is None)
            assert_equal(len(topics.providers), 1)

            # topic is very likely not detected yet ( we didn't wait after creating and exposing it )
            res = topics.call()
            assert_true(not 'test_topic' in res)

            time.sleep(mockn.update_interval + 1)  # make sure we let update time to kick in

            # topic has to be detected now
            res = topics.call()
            assert_true('test_topic' in res)

    finally:  # to make sure we clean up on failure
        mockn.shutdown()
        assert_false(mockn.is_alive())


def test_mocknode_subscribers_detect():  # Here we check that this node actually detects a topic
    mockn = PyrosMock(kwargs={
        'services': [],
        'publishers': [],
        'subscribers': ['test_topic'],
        'params': []
    })
    assert_false(mockn.is_alive())

    assert_true(hasattr(mockn, 'topics'))

    # starting the node
    mockn.start()

    # checking interface is still None here ( instantiated in child only )
    assert_true(mockn.interface is None)

    # Services are initialized in run() method of pyzmp.Node, after interface has been initialized
    try:
        assert_true(mockn.is_alive())

        with mock_subscriber_remote('test_topic', statusecho_topic):

            # asserting the mock system has done its job from our point of view at least
            assert_true('test_topic' in topics_available_remote)
            assert_equal(topics_available_type_remote['test_topic'], statusecho_topic)

            # Getting topics list from child process
            print("Discovering topics Service...")
            topics = pyzmp.discover("topics", 3)  # we wait a bit to let it time to start
            assert_false(topics is None)
            assert_equal(len(topics.providers), 1)

            time.sleep(mockn.update_interval + 1)  # make sure we let update time to kick in

            res = topics.call(recv_timeout=6000000)
            # the mock system should have done its job from the other process perspective too
            # via multiprocess manager list
            assert_true('test_topic' in res)  # topic detected since in list of exposed topics

    finally:
        mockn.shutdown()
        assert_false(mockn.is_alive())


def test_mocknode_subscribers_detect_setup():  # Here we check that this node actually detects a topic upon setup
    mockn = PyrosMock()
    assert_false(mockn.is_alive())

    assert_true(hasattr(mockn, 'topics'))

    mockn.start()
    try:
        assert_true(mockn.is_alive())

        with mock_subscriber_remote('test_topic', statusecho_topic):

            print("Discovering topics Service...")
            topics = pyzmp.discover("topics", 3)  # we wait a bit to let it time to start
            assert_false(topics is None)
            assert_equal(len(topics.providers), 1)

            res = topics.call()
            assert_true(not 'test_topic' in res)  # topic not detected since not in list of exposed topics

            print("Discovering setup Service...")
            setup = pyzmp.discover("setup", 3)  # we wait a bit to let it time to start
            assert_false(setup is None)
            assert_equal(len(setup.providers), 1)

            setup.call(kwargs={'services': [], 'topics': ['test_topic'], 'params': []})

            time.sleep(mockn.update_interval + 1)  # waiting for update to kick in

            res = topics.call()
            assert_true('test_topic' in res)
    finally:
        mockn.shutdown()
        assert_false(mockn.is_alive())


def test_mocknode_subscribers_detect_throttled():
    """
    Testing that the mocknode detection of topics is throttled properly
    :return:
    """
    mockn = PyrosMock()
    assert_false(mockn.is_alive())

    assert_true(hasattr(mockn, 'topics'))

    mockn.update_interval = 5  # we wait 5 seconds between each update_throttled call
    mockn.start()  # one update will be triggered, and then nothing for the next 10 seconds
    try:
        assert_true(mockn.is_alive())

        print("Discovering setup Service...")
        setup = pyzmp.discover("setup", 3)  # we wait a bit to let it time to start
        assert_false(setup is None)
        assert_equal(len(setup.providers), 1)

        setup.call(kwargs={'services': [], 'topics': ['test_topic'], 'params': []})

        with mock_subscriber_remote('test_topic', statusecho_topic):

            print("Discovering topics Service...")
            topics = pyzmp.discover("topics", 3)  # we wait a bit to let it time to start
            assert_false(topics is None)
            assert_equal(len(topics.providers), 1)

            # topic is very likely not detected yet ( we didn't wait after creating and exposing it )
            res = topics.call()
            assert_true(not 'test_topic' in res)

            time.sleep(mockn.update_interval + 1)  # make sure we let update time to kick in

            # topic has to be detected now
            res = topics.call()
            assert_true('test_topic' in res)

    finally:  # to make sure we clean up on failure
        mockn.shutdown()
        assert_false(mockn.is_alive())


def test_msg_build():
    msg = PyrosMock().msg_build('fake_connec_name')
    print("msg is of type {0}".format(type(msg)))
    assert isinstance(msg, str)


def test_echo_topic_default():
    mock = PyrosMock()
    recv_msg = mock.topic('random_topic')
    assert recv_msg is None


def test_echo_same_topic():
    msg = 'testing'
    mock = PyrosMock()
    mock.topic('random_topic', msg)
    print("msg sent is {0}".format(msg))
    recv_msg = mock.topic('random_topic')
    print("msg received is {0}".format(recv_msg))
    assert msg == recv_msg


def test_other_topic():
    msg = 'testing'
    mock = PyrosMock()
    mock.topic('random_topic', msg)
    print("msg sent is {0}".format(msg))
    recv_msg = mock.topic('random_topic_2')
    print("msg received is {0}".format(recv_msg))
    assert recv_msg is None


def test_echo_service_default():
    msg = 'testing'
    mock = PyrosMock()
    assert mock.service('random_service') is None


def test_echo_service():
    msg = 'testing'
    mock = PyrosMock()
    print("msg sent is {0}".format(msg))
    recv_msg = mock.service('random_service', msg)
    print("msg received is {0}".format(recv_msg))
    assert msg == recv_msg


class TestPyrosMockProcess(object):
    def setUp(self):
        self.mockInstance = PyrosMock()
        self.mockInstance.start()

    def tearDown(self):
        self.mockInstance.shutdown()

    def test_msg_build(self):
        msg_build_svc = pyzmp.Service.discover('msg_build', 5)
        assert(msg_build_svc is not None)
        assert(self.mockInstance not in msg_build_svc.providers)
        resp = msg_build_svc.call(args=('fake_connec_name',))
        assert isinstance(resp, str)

    def test_list_topic(self):
        list_topic_svc = pyzmp.Service.discover('topics', 5)
        assert(list_topic_svc is not None)
        assert(self.mockInstance not in list_topic_svc.providers)
        resp = list_topic_svc.call()
        assert resp is not None

    def test_echo_topic(self):
        topic_svc = pyzmp.Service.discover('topic', 5)
        assert(topic_svc is not None)
        assert(self.mockInstance not in topic_svc.providers)
        resp = topic_svc.call(args=('random_topic', 'testing'))
        assert resp is None  # message consumed

        resp = topic_svc.call(args=('random_topic', None))
        assert resp == 'testing'  # message echoed

    def test_other_topic(self):
        topic_svc = pyzmp.Service.discover('topic', 5)
        assert(topic_svc is not None)
        assert(self.mockInstance not in topic_svc.providers)
        resp = topic_svc.call(args=('random_topic', 'testing'))
        assert resp is None  # message consumed

        resp = topic_svc.call(args=('random_topic_2', None))
        assert resp is None  # message not echoed

    def test_list_service(self):
        service_svc = pyzmp.Service.discover('services', 5)
        assert(service_svc is not None)
        assert(self.mockInstance not in service_svc.providers)
        resp = service_svc.call()
        assert resp is not None  # message echoed

    def test_echo_service(self):
        service_svc = pyzmp.Service.discover('service', 5)
        assert(service_svc is not None)
        assert(self.mockInstance not in service_svc.providers)
        resp = service_svc.call(args=('random_service', 'testing'))
        assert resp == 'testing'  # message echoed

    #TODO Check that if a service is called inappropriately, the exception is properly transferred back to the calling process and reraised.

if __name__ == '__main__':

    import nose
    nose.runmodule()



# -*- coding: utf-8 -*-
# This python package is handling all MOCK communication for rostful-node.
# The point of it is to be able to fully tests the multiprocess behavior,
#     in pure python, without having to run a ROS system.
#
# If possible these should also be base classes for all the possible multiprocess framework interfaces
# available in pyros
#
# Mock interface is provided with the package, because it may help client libraries to implement their own tests
#
from __future__ import absolute_import

from .mockinterface import MockInterface
from .mockmessage import (
    extract_values,
    populate_instance,
    FieldTypeMismatchException,
    NonexistentFieldException,
    StatusMsg,
)
from .mockparam import MockParam
from .mockservice import MockService, statusecho_service
from .mocksubscriber import MockSubscriber
from .mockpublisher import MockPublisher
from .mocksystem import MockSystem, statusecho_topic


# Since this mock interface follows ROS design (params, service, topics)
# We decided to make it part of ROS interface

# TODO : We should probably use mock library for this and test against it...

__all__ = [
    'MockPublisher',
    'MockSubscriber',
    'MockService',
    'MockParam',
    'MockSystem',
    'extract_msg',
    'populate_msg',
    'FieldTypeMismatchException',
    'NonexistentFieldException',
    'StatusMsg',
    'statusecho_topic',
    'statusecho_service',
    'MockInterface',
]



# Because rospy :
# - expects OS state to be very special
# - modify global python interpreter state when using node_init ( which is required to receive message on topics )
# => Communication features are only testable with rostest.
# However, running them one by one with nosetest works (One run, one test module, one process).
# This allows easy debugging ( even during start / stop sequences )
# Therefore, tests are executable so that nosetests doesn't auto detect them.

import time


class Timeout(object):
    """
    Small useful timeout class
    """
    def __init__(self, seconds):
        self.seconds = seconds

    def __enter__(self):
        self.die_after = time.time() + self.seconds
        return self

    def __exit__(self, type, value, traceback):
        pass

    @property
    def timed_out(self):
        return time.time() > self.die_after

# reused fixtures:
from .testRosInterface import TestRosInterface



from __future__ import absolute_import

from collections import deque, namedtuple

from ...baseinterface import TransientIf

ServiceType = namedtuple("ServiceType", "reqtype resptype")

statusecho_service = ServiceType("StatusMsg", "StatusMsg")


class MockService(TransientIf):
    """
    MockService is a mock for the class handling conversion from python API to Service call
    """
    def __init__(self, service_name, service_type):
        # setting the name to make sure we start with /
        service_name = service_name if service_name.startswith('/') else '/' + service_name

        # service_type is a composition of python standard builtin types that still make sense outside of
        # the python environment where there originated from :
        # int, float, long, complex, str, unicode, list, tuple, bytearray, buffer, xrange, set, frozenset, dict
        service_type = (service_type.reqtype, service_type.resptype)

        # Initializing the transient
        super(MockService, self).__init__(service_name, service_type)

    def cleanup(self):
        pass

    def call(self, rosreq=None):
        # simulating echo

        return rosreq

